import { IParam, PickupView } from "@core";
import { MPickupViewProps } from "./mpickup-view-prop";
import { MPickupViewState } from "./mpickup-view-state";

/**
 * @description 多选视图
 * @export
 * @class MPickupView
 * @extends {PickupView}
 */
export class MPickupView extends PickupView {

  /**
   * @description 视图状态
   * @type {MPickupViewState}
   * @memberof PickupView
   */
  public declare state: MPickupViewState;

  /**
   * @description 视图输入参数
   * @type {MPickupViewProps}
   * @memberof MPickupView
   */
  public declare props: MPickupViewProps;

  public select: IParam = {};

  public onMoveRight() {
    const { selections, selectedData } = toRefs(this.state);
    selections.value.push(selectedData.value);
  }

  public onMoveLeft() {
    const { selections } = toRefs(this.state);
    const index = selections.value.findIndex((selection: any) => Object.is(this.select.srfkey, selection.srfkey));
    if (index !== -1) {
      selections.value.splice(index, 1);
    }
  }

  public onAllMoveRight() {
    //todo
  }

  public onAllMoveLeft() {
    const { selections } = toRefs(this.state);
    selections.value.splice(0, selections.value.length);
  }

  /**
   * @description 安装视图所有功能模块的方法
   * @return {*} 
   * @memberof PickupView
   */
  public moduleInstall() {
    const superParams = super.moduleInstall();
    return {
      ...superParams,
      onMoveRight: this.onMoveRight.bind(this),
      onMoveLeft: this.onMoveLeft.bind(this),
      onAllMoveRight: this.onAllMoveRight.bind(this),
      onAllMoveLeft: this.onAllMoveLeft.bind(this),
    };
  }
}